/*
 * Lights.h
 *
 *  Created on: Feb 19, 2009
 *      Author: njoubert
 */

#ifndef LIGHTS_H_
#define LIGHTS_H_

#include "global.h"

/*
 * All the different types of lights inherit from this base class, which
 * only has the property of color.
 */
class Light {
protected:

    Light();
    Light(RGB illumination);
    RGB _illumination;

public:

    virtual ~Light();

    const RGB & getColor();
    void setColor(RGB c);

    /**
     * Returns the normalized vector describing the direction of light falling
     * onto given position. This is used in the shading calculation.
     */
    virtual void getIncidenceVector(vec4 & position, vec4 & returnValue)=0;

};

class AmbientLight : public Light {
public:
    AmbientLight();
    AmbientLight(RGB illumination);
    void getIncidenceVector(vec4 & position, vec4 & returnValue);
};

class PointLight : public Light {
public:
    PointLight(RGB illumination);
    void setPosition(vec4 pos);
    void getIncidenceVector(vec4 & position, vec4 & returnValue);
private:
    vec4 _pos;
};

class DirectionalLight : public Light {
public:
    DirectionalLight(RGB illumination);
    void setDirection(vec4 dir);
    void getIncidenceVector(vec4 & position, vec4 & returnValue);
private:
    vec4 _dir;
};

#endif /* LIGHTS_H_ */
