/*
 * Viewport.cpp
 *
 *  Created on: Feb 19, 2009
 *      Author: njoubert
 */

#include "Viewport.h"

Viewport::Viewport(vec4 eye, vec4 LL, vec4 UL, vec4 LR, vec4 UR, int width, int height) {
    _eye = eye;
    _LL = LL;
    _UL = UL;
    _LR = LR;
    _UR = UR;
    _pixelsWide = width;
    _pixelsHigh = height;
}

Viewport::~Viewport() {

}

void Viewport::resetSampler() {
    _i = _j = 0.5; //Starts off at the center of a pixel.
}

/**
 * This ONLY fills in the (i, j) coordinates of the point according
 * to the pixels_wide and pixels_high settings
 */
bool Viewport::getSample(Point & p) {
    _i += 1;
    if (_i > _pixelsWide) {
        _i = 0.5;
        _j += 1;
    }
    if (_j > _pixelsHigh) {
        resetSampler();
        return false;
    }

    double u = _i / (double) _pixelsWide;
    double v = _j / (double) _pixelsHigh;

    p[0] = ((1-u)*((1-v)*_LL[0] + v*_UL[0]) + u*((1-v)*_LR[0] + v*_UR[0]));
    p[1] = ((1-u)*((1-v)*_LL[1] + v*_UL[1]) + u*((1-v)*_LR[1] + v*_UR[1]));
    p[2] = ((1-u)*((1-v)*_LL[2] + v*_UL[2]) + u*((1-v)*_LR[2] + v*_UR[2]));

    p.i() = _i; //the x-pixel coordinate
    p.j() = _j; //the y-pixel coordinate

    return true;
}
int Viewport::getW() { return _pixelsWide; }

int Viewport::getH() { return _pixelsHigh; }

